package com.platform.yijia.controller;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.platform.yijia.param.request.AppUserInfoRequest;
import com.platform.yijia.pojo.AppUserInfo;
import com.platform.yijia.pojo.CustomerManage;
import com.platform.yijia.pojo.CustomerPoints;
import com.platform.yijia.service.AppUserInfoService;
import com.platform.yijia.service.CustomerPointsService;
import com.platform.yijia.service.PayOrderService;
import com.platform.yijia.service.StationService;
import com.platform.yijia.utils.CodeMsg;
import com.platform.yijia.utils.ResultData;
import com.platform.yijia.utils.redis.RedisCacheUtil;
import com.platform.yijia.utils.weixinapp.AesDecryptUtil;
import com.platform.yijia.utils.weixinapp.WeiXinUserUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Controller
@RequestMapping("/demo")
public class AppUserInfoController {
    private static Logger logger =(Logger) LoggerFactory.getLogger(AppUserInfoController.class);
    @Autowired
    private PayOrderService payOrderService;
    @Autowired
    private AppUserInfoService appUserInfoService;
    @Resource
    private StationService stationService;
    @Resource
    private CustomerPointsService customerPointsService;
    @Resource
    private RedisCacheUtil redisCacheUtil;

//    String appId = "wxe1135cd390b38a54";        //微信小程序appID
//    String appSecret = "0532c7d9ae876c4ad636df0b1e3b9ddb";      //微信小程序密钥

    /*
     * 微信小程序根据code获取用户openId和sessionKey
     * @param code
     * @param stationId
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/getSessionKeyAndOpenID")
    public Map<String, Object> getSessionKeyAndOpenID(@RequestParam("code") String code, @RequestParam("stationId") Integer stationId){
        logger.info("微信小程序获取用户openId和sessionKey入参code和stationId: "+code +"-----" +stationId);
        String appId = "";        //微信小程序appID
        String appSecret = "";      //微信小程序密钥
        if(stationId !=null){
            Map<String, String> map = stationService.getStationAppIdAndAppSecret(stationId);
            appId = map.get("appId");        //微信小程序appID
            appSecret = map.get("appSecret");      //微信小程序密钥
        }
        //获取session_key 和 openid
        return WeiXinUserUtil.getUserSessionKeyAndOpenID(appId, appSecret, code);
    }

    /*
     * 解密 encryptedData
     * @param sessionKey
     * @param encryptedData
     * @param iv
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/decryptEncryptedData", method = RequestMethod.POST, consumes = "application/json")
    public String decryptEncryptedData(@RequestBody JSONObject jsonObject){
        logger.info("解密获取手机号前台传入参数：" +jsonObject.toString());
        return AesDecryptUtil.decryptEncryptedData(jsonObject.get("encryptedData").toString(), jsonObject.get("sessionKey").toString(), jsonObject.get("iv").toString(), "UTF-8").toString();
    }

    //判断使用小程序支付的用户是否关注公众号
    @RequestMapping(value = "/whetherFollowGzh", method = RequestMethod.GET)
    @ResponseBody
    public String whetherFollowGzh(@RequestParam String unionId, Integer stationId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if (unionId !=null){
            Map<String, String> param = new HashMap<>();
            param.put("unionId", unionId);
            param.put("userType", "1");
            String openId = appUserInfoService.whetherFollowGzh(param);
            if(openId !=null){
                Map<String, String> m = stationService.getStationAppIdAndAppSecret(stationId);
                String gzhAppId ="";
                String gzhAppSecret ="";
                String accessToken ="";
                if(m !=null && m.containsKey("gzhAppId") && m.containsKey("gzhAppSecret")){
                    gzhAppId = m.get("gzhAppId");
                    gzhAppSecret = m.get("gzhAppSecret");
                    accessToken  = this.getTokenByRedisCache(gzhAppId, gzhAppSecret);
                    net.sf.json.JSONObject unionIdInfo = WeiXinUserUtil.getUnionIdInfo(accessToken, openId);
                    if(unionIdInfo.containsKey("subscribe") && unionIdInfo.get("subscribe").equals("1")){
                        resultData = ResultData.success(CodeMsg.SUCCESS);
                    }else {
                        resultData = ResultData.error(CodeMsg.NO_FollowGzh);
                    }
                }
            }else {
                resultData = ResultData.error(CodeMsg.USERGZHINFO_EXSIST);
            }
        }
        return gson.toJson(resultData);
    }

    /*
     * 获取公众号授权后的信息
     * @param code
     * @param stationId
     * @return
     */
    @RequestMapping(value = "/getGzhAuthorizeInfo", method = RequestMethod.GET)
    @ResponseBody
    public String getGzhAuthorizeInfo(@RequestParam String code, Integer stationId){
        logger.info("获取公众号授权后的信息前端传递参数：code："+ code );
        logger.info("获取公众号授权后的信息前端传递参数：stationId：" +stationId);
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(code != null && stationId !=null){
            String gzhAppId ="";
            String gzhAppSecret ="";
            String accessToken ="";
            Map<String, String> m = stationService.getStationAppIdAndAppSecret(stationId);
            if(m !=null && m.containsKey("gzhAppId") && m.containsKey("gzhAppSecret")){
                gzhAppId = m.get("gzhAppId");
                gzhAppSecret = m.get("gzhAppSecret");
                accessToken  = this.getTokenByRedisCache(gzhAppId, gzhAppSecret);
                net.sf.json.JSONObject gzhAuthorizeInfo = WeiXinUserUtil.getGzhAuthorizeInfo(gzhAppId, gzhAppSecret, accessToken);
                resultData = ResultData.success(gzhAuthorizeInfo);
            }
        }else {
            resultData = ResultData.success(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }


    //获取公众号信息
    @RequestMapping(value = "/getGzhUserInfo", method = RequestMethod.GET)
    @ResponseBody
    public String getGzhUserInfo(@RequestParam String openId, Integer stationId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        //Map<String, String> map= new HashMap<>();
        if(openId !=null && stationId !=null){
            String gzhAppId ="";
            String gzhAppSecret ="";
            String accessToken ="";
            Map<String, String> m = stationService.getStationAppIdAndAppSecret(stationId);
            if(m !=null && m.containsKey("gzhAppId") && m.containsKey("gzhAppSecret")){
                gzhAppId = m.get("gzhAppId");
                gzhAppSecret = m.get("gzhAppSecret");
                accessToken  = this.getTokenByRedisCache(gzhAppId, gzhAppSecret);
                net.sf.json.JSONObject unionIdInfo = WeiXinUserUtil.getUnionIdInfo(accessToken, openId);
                resultData=ResultData.success(unionIdInfo);
            }
        }else {
            resultData=ResultData.success(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    /*
     * 从Redis缓存中获取 tokenCache
     * @param appId
     * @param appSecret
     * @return
     */
    public String getTokenByRedisCache(String appId, String appSecret){
        logger.info("appId参数: "+ appId+" ;appSecret参数: " +appSecret);
        String tokenCache ="";
        if(!redisCacheUtil.hasKey(appId)){
            String token = WeiXinUserUtil.getToken(appId, appSecret).getAccessToken();
            redisCacheUtil.setCacheObject(appId, token);
            redisCacheUtil.expire(appId, 7200, TimeUnit.SECONDS);
            tokenCache = token;
            logger.info("Redis缓存中token信息: " + tokenCache);
        }else {
            tokenCache = redisCacheUtil.getCacheObject(appId);
        }
        return tokenCache;
    }

    /*
     * 添加用户信息
     * @param request
     * @return
     */
    @RequestMapping(value = "/addAppUserInfo", method = RequestMethod.POST)
    @ResponseBody
    @Transactional
    public String addAppUserInfo(@RequestBody AppUserInfoRequest request){
        Gson gson =new Gson();
        ResultData resultData=null;
        try{
            logger.info("前端传入参数：" +request.toString());
            AppUserInfo appUserInfo =new AppUserInfo();
            CustomerPoints customerPoints =  new CustomerPoints();
            String openId = "";     //此参数用于查询该用户是否已注册过
            if(request.getUserType()!=null){
                //1-微信公众号
                if(request.getUserType().equals("1") &&StringUtils.isNotBlank(request.getOpenId())){
                    appUserInfo.setUserType(request.getUserType());
                    appUserInfo.setBlogToken(request.getBlogToken());
                    openId = request.getOpenId();
                    appUserInfo.setBlogOpenid(openId);
                    customerPoints.setBlogOpenId(openId);
                 //2-微信小程序
                }else if(request.getUserType().equals("2") &&StringUtils.isNotBlank(request.getOpenId())){
                    appUserInfo.setUserType(request.getUserType());
                    appUserInfo.setMinaToken(request.getMinaToken());
                    openId = request.getOpenId();
                    appUserInfo.setMinaOpenid(openId);
                    customerPoints.setMinaOpenId(openId);
                }
            }
            if(StringUtils.isNotBlank(request.getBlogNickName())){
                appUserInfo.setBlogNickName(request.getBlogNickName());
                customerPoints.setCustomerName(request.getBlogNickName());
            }
            if(StringUtils.isNotBlank(request.getMobilePhone())){
                appUserInfo.setMobilePhone(request.getMobilePhone());
                customerPoints.setMobilePhone(request.getMobilePhone());
            }
            if(StringUtils.isNotBlank(request.getBlogProfilePhoto())){
                appUserInfo.setBlogProfilePhoto(request.getBlogProfilePhoto());
            }
            if(StringUtils.isNotBlank(request.getUnionId())){
                appUserInfo.setUnionId(request.getUnionId());
                customerPoints.setUnionId(request.getUnionId());
            }

            appUserInfo.setRegisteDate(new Date());
            appUserInfo.setUpdateDate(new Date());

            customerPoints.setCreateTime(new Date());
            customerPoints.setUnionId(request.getUnionId());
            customerPoints.setPoints(0);
            customerPoints.setConsumptionPoints(0);
            customerPoints.setAccumulatePoints(0);
            customerPoints.setInvalidPoints(0);
            customerPoints.setStationId(request.getStationId());
            customerPoints.setStationName(request.getStationName());

            if(StringUtils.isNotBlank(request.getSexFlag())){
                appUserInfo.setSexFlag(request.getSexFlag());
            }
            logger.info("插入数据库用户信息参数："+appUserInfo.toString());
            Map<String, Object> params = new HashMap<>();
            params.put("openId", openId);                   //注册用户openId
            params.put("userType", request.getUserType());  //用户类型
            //判断该用户是否已存在
            int count = appUserInfoService.isExistAppUser(params);
            if(count > 0){
                //此处是为初期没使用unionid，以作更新前期用户unionId
                String userUnionId = appUserInfoService.getUserUnionId(appUserInfo);
                if(!StringUtils.isNotBlank(userUnionId)){
                    appUserInfoService.updateUnionId(appUserInfo);
                }
                this.upDateCustomerPointsInfo(customerPoints, request);
                resultData = ResultData.error(CodeMsg.USER_EXSIST);
            }else {
                appUserInfoService.AddAppUserInfo(appUserInfo);
                this.upDateCustomerPointsInfo(customerPoints, request);
                resultData = ResultData.success(CodeMsg.SUCCESS);
            }
        }catch (Exception e){
            resultData = ResultData.error(CodeMsg.INSERT_FAIL);
            e.printStackTrace();
        }
        return gson.toJson(resultData);
    }

    //客户积分表信息操作
    public void upDateCustomerPointsInfo(CustomerPoints customerPoints, AppUserInfoRequest request){
        boolean boo = customerPointsService.isExistCustomerPointsInfo(customerPoints);
        if(!boo){
            customerPointsService.insertCustomerPointsInfo(customerPoints);
        }else {
            //此处用于：如果客户积分信息存在可能会出现小程序ID或者公众号ID没有的情况
            CustomerPoints c = new CustomerPoints();
            c.setUnionId(customerPoints.getUnionId());
            if(request.getUserType().equals("1")){
                c.setBlogOpenId(customerPoints.getBlogOpenId());
            }else if(request.getUserType().equals("2")){
                c.setMinaOpenId(customerPoints.getMinaOpenId());
            }
            c.setStationId(request.getStationId());
            c.setStationName(request.getStationName());
            c.setMobilePhone(request.getMobilePhone());
            customerPointsService.updateCustomerPointsInfo(c);
        }
    }

}
